// Ocean implementation
// Copyright (C) 05.09.97 Marco Dettori and Oliver Obst
//
// $Id: ocean.C,v 0.2 1998/03/16 10:33:34 oliver Exp $

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

#include "ocean.h"

// --- constructor ------------------------------------------------------------

Ocean :: Ocean( char side, char ships, char *lengths, char stlen,
                QWidget *parent, const char *name )
        : QFrame( parent, name )
{
        TriState *tb;
        
        edge    = side;
        size    = edge * edge;

        buttons = new TriStates(size);
        
        for (int i=0; i < size; i++) {
                tb = new TriState(this);
                connect(tb, SIGNAL(clicked()), SLOT(handleClick()));
                buttons->insert(i, tb);
        }
        
        st = new Status(ships, lengths, stlen, this);
}

// --- destructor -------------------------------------------------------------
        
Ocean :: ~Ocean()
{
        delete buttons;
}

// --- public -----------------------------------------------------------------
// Ocean :: setBackgroundColor(QColor &color)

void Ocean :: setBackgroundColor(const QColor &color)
{
        st->setBackgroundColor( color );
        QWidget::setBackgroundColor( color );
}


// --- public -----------------------------------------------------------------
// Ocean :: surround
//
// surround a ship with water and return the length of the ship

int Ocean :: surround( int id )
{
        int     l = 1;
        int     line = int( id / edge ) * edge;
        int     i;

        i = 1;
        while  (id - i >= line &&
                buttons->at(id - i)->getState() == TriState::CROSS) {
                if ( id - i - edge >= 0 )
                        buttons->at(id - i - edge)->setState(TriState::CIRCLE);
                if (id - i + edge < size)
                        buttons->at(id - i + edge)->setState(TriState::CIRCLE);
                l++;
                i++;
        }
        if ( id - i >= line )
                buttons->at(id - i)->setState( TriState::CIRCLE );

        i = 1;
        while (id + i < line + edge &&
               buttons->at(id + i)->getState() == TriState::CROSS ) {
                if ( id + i - edge >= 0 )
                        buttons->at(id + i - edge)->setState(TriState::CIRCLE);
                if ( id + i + edge < size )
                        buttons->at(id + i + edge)->setState(TriState::CIRCLE);
                l++;
                i++;
        }
        if ( id + i < line + edge )
                buttons->at(id + i)->setState( TriState::CIRCLE );

        i = edge;
        while  ( id - i >= 0 &&
                 buttons->at(id - i)->getState() == TriState::CROSS ) {
                if ( id - 1 >=  line )
                        buttons->at(id - i - 1)->setState(TriState::CIRCLE);
                if ( id + 1 < line + edge )
                        buttons->at(id - i + 1)->setState(TriState::CIRCLE);
                l++;
                i += edge;
        }
        if ( id - i >= 0 )
                buttons->at(id - i)->setState( TriState::CIRCLE );

        i = edge;
        while  ( id + i < size &&
                 buttons->at(id + i)->getState() == TriState::CROSS ) {
                if ( id - 1 >=  line )
                        buttons->at(id + i - 1)->setState(TriState::CIRCLE);
                if ( id + 1 < line + edge )
                        buttons->at(id + i + 1)->setState(TriState::CIRCLE);
                l++;
                i += edge;
        }
        if ( id + i < size )
                buttons->at(id + i)->setState(TriState::CIRCLE);


        return l;
}                

// --- protected --------------------------------------------------------------
// Ocean :: resizeEvent( QResizeEvent * )
//
// resize the whole thing and adjust the button sizes

void Ocean :: resizeEvent( QResizeEvent * )
{
        int osize       = width() - 2;
        
        if ( height() < width() )
                osize   = height() - 2;

        int bsize       = osize / edge;
        int border      = ( width() - bsize * edge ) / 2;
        int ssize       = height() - 2 * border - osize - 4;
        
        for (int i=0; i < edge; i++ )
                for (int j=0; j < edge; j++ )
                        buttons->at(j * edge + i)
                                ->setGeometry(bsize * i + border,
                                              bsize * j + border,
                                              bsize, bsize );
        
        st->setGeometry( border, osize + border + 2, bsize * edge, ssize );
}

// --- public ----------------------------------------------------------------
// Ocean :: reset()
//
// clear the whole square

void Ocean :: reset()
{
        for (int i=0; i < size; i++ )
                buttons->at(i)->setState( TriState::EMPTY );

        st->resetStatus();
}


// --- protected slot --------------------------------------------------------
// Ocean :: handleClick()
//
// what to do if a button is pressed 
//      since this has to be done in child classes, this code here
//      is only an example.

void Ocean :: handleClick()
{
        int id = buttons->findRef((TriState *) sender());        
        TriState::State s = buttons->at(id)->getState();

        switch ( s ) {
        case TriState::EMPTY :
                buttons->at(id)->setState( TriState::CIRCLE );
                break;
        case TriState::CIRCLE :
                buttons->at(id)->setState( TriState::CROSS );
                break;
        case TriState::CROSS :
                buttons->at(id)->setState( TriState::EMPTY );
        }
}
