// OpOcean implementation
// Copyright (C) 01.09.97 Marco Dettori and Oliver Obst
//
// $Id: opocean.C,v 0.2 1998/03/16 10:35:32 oliver Exp $

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

#include <iostream.h>

#include "opocean.h"
#include "protoconst.h"

// --- constructor ------------------------------------------------------------

OpOcean :: OpOcean( char sql, char ships, char *lengths, char stlen,
                    QWidget *parent, const char *name )
        : Ocean( sql, ships, lengths, stlen, parent, name )
{
        ship    = 0;
        num     = ships;
        state   = WAITING;
}

// --- public -----------------------------------------------------------------
// OpOcean :: initState
//
// set internal state

void OpOcean :: setState( State s )
{
        state = s;
}


// --- public -----------------------------------------------------------------
// OpOcean :: getState
//
// get internal state

OpOcean::State OpOcean :: getState()
{
        return state;
}


// --- public slot ------------------------------------------------------------
// OpOcean :: report( int id, int result )
//              receive a result from opponent and mark the field according
//              to that. set internal state.

void OpOcean :: report( int id, int result ) 
{
        switch( result ) {
        case MISSED :
                buttons->at(id)->setState( TriState::CIRCLE );
                state = OPTURN;
                break;
        case HIT :
                buttons->at(id)->setState( TriState::CROSS );
                state = MYTURN;
                break;
        default  :
                buttons->at(id)->setState( TriState::CROSS );
                state = MYTURN;

                int id2 = id;
                int line = int( id / edge ) * edge;
                
                if (id - 1 >= line &&
                    buttons->at(id - 1)->getState() == TriState::CROSS)
                        id2 = id - 1;

                if ( id + 1 < line + edge &&
                     buttons->at(id + 1)->getState() == TriState::CROSS )
                        id2 = id + 1;

                if ( id - edge >= 0 &&
                     buttons->at(id - edge)->getState() == TriState::CROSS )
                        id2 = id - edge;

                if ( id + edge < size &&
                     buttons->at(id + edge)->getState() == TriState::CROSS )
                        id2 = id + edge;

                st->setShip(surround(id));
                
                if ( ++ship == num )
                        state = WAITING;
        }
}

// --- public slot ------------------------------------------------------------
//
// OpOcean :: reset()

void OpOcean :: reset()
{
        for (int i=0; i < size; i++ ) {
                buttons->at(i)->setState( TriState::EMPTY );
                buttons->at(i)->setBackgroundColor( blue );
                buttons->at(i)->visit( FALSE );
        }
        st->resetStatus();
        ship    = 0;
        state   = WAITING;
}


// --- protected slot --------------------------------------------------------
// OpOcean :: handleClick
//
// what to do if a button is pressed 

void OpOcean :: handleClick()
{
        int id = buttons->findRef((TriState *) sender());
        TriState::State s = buttons->at(id)->getState();

        if (( state == MYTURN ) && ( s == TriState::EMPTY )) {
                state   = WAITING;
                emit request(id);
        }
}
